@@ -1013,11 +1013,6 @@ static PyObject *WraptObjectProxy_getattro(
 {
     PyObject *object = NULL;
 
-    if (!self->wrapped) {
-      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialised");
-      return NULL;
-    }
-
     object = PyObject_GenericGetAttr((PyObject *)self, name);
 
     if (object)
@@ -1025,6 +1020,11 @@ static PyObject *WraptObjectProxy_getattro(
 
     PyErr_Clear();
 
+    if (!self->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialised");
+      return NULL;
+    }
+
     return PyObject_GetAttr(self->wrapped, name);
 }
 
@@ -1040,11 +1040,6 @@ static int WraptObjectProxy_setattro(
 
     PyObject *match = NULL;
 
-    if (!self->wrapped) {
-      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialised");
-      return -1;
-    }
-
 #if PY_MAJOR_VERSION >= 3
     self_prefix = PyUnicode_FromString("_self_");
 #else
@@ -1077,6 +1072,11 @@ static int WraptObjectProxy_setattro(
 
     Py_DECREF(attr_wrapped);
 
+    if (!self->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialised");
+      return -1;
+    }
+
 #if PY_MAJOR_VERSION >= 3
     attr_name = PyUnicode_FromString("__name__");
     attr_qualname = PyUnicode_FromString("__qualname__");
@@ -2044,6 +2044,16 @@ moduleinit(void)
     PyModule_AddObject(module, "FunctionWrapper",
             (PyObject *)&WraptFunctionWrapper_Type);
 
+    Py_INCREF(&WraptFunctionWrapperBase_Type);
+    PyModule_AddObject(module, "_FunctionWrapperBase",
+            (PyObject *)&WraptFunctionWrapperBase_Type);
+    Py_INCREF(&WraptBoundFunctionWrapper_Type);
+    PyModule_AddObject(module, "_BoundFunctionWrapper",
+            (PyObject *)&WraptBoundFunctionWrapper_Type);
+    Py_INCREF(&WraptBoundMethodWrapper_Type);
+    PyModule_AddObject(module, "_BoundMethodWrapper",
+            (PyObject *)&WraptBoundMethodWrapper_Type);
+
     return module;
 }
 
